@@ -21,12 +21,24 @@ package com.eteks.furniturelibraryeditor;
 
 import java.awt.EventQueue;
 import java.awt.Image;
+import java.awt.desktop.AboutEvent;
+import java.awt.desktop.AboutHandler;
+import java.awt.desktop.OpenFilesEvent;
+import java.awt.desktop.OpenFilesHandler;
+import java.awt.desktop.PreferencesEvent;
+import java.awt.desktop.PreferencesHandler;
+import java.awt.desktop.QuitEvent;
+import java.awt.desktop.QuitHandler;
+import java.awt.desktop.QuitResponse;
+import java.awt.desktop.QuitStrategy;
 import java.awt.event.WindowAdapter;
 import java.awt.event.WindowEvent;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.io.File;
 import java.io.IOException;
+import java.lang.reflect.InvocationTargetException;
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
 import java.util.ResourceBundle;
@@ -43,6 +55,7 @@ import javax.swing.JRootPane;
 import javax.swing.KeyStroke;
 import javax.swing.SwingUtilities;
 import javax.swing.UIManager;
+import javax.swing.filechooser.FileFilter;
 
 import com.apple.eawt.Application;
 import com.apple.eawt.ApplicationAdapter;
@@ -77,13 +90,13 @@ public class FurnitureLibraryEditor {
    * Returns a recorder able to write and read furniture library files.
    */
   public FurnitureLibraryRecorder getFurnitureLibraryRecorder() {
-    // Initialize furnitureLibraryRecorder lazily 
+    // Initialize furnitureLibraryRecorder lazily
     if (this.furnitureLibraryRecorder == null) {
       this.furnitureLibraryRecorder = new FurnitureLibraryFileRecorder();
     }
     return this.furnitureLibraryRecorder;
   }
-  
+
   /**
    * Returns user preferences stored in resources and local file system.
    */
@@ -94,7 +107,7 @@ public class FurnitureLibraryEditor {
     }
     return this.userPreferences;
   }
-  
+
   /**
    * Returns a content manager able to handle files.
    */
@@ -102,7 +115,7 @@ public class FurnitureLibraryEditor {
     if (this.contentManager == null) {
       this.contentManager = new FileContentManager(getUserPreferences()) {
           private File modelsDirectory;
-  
+
           @Override
           public String showOpenDialog(View parentView,
                                      String dialogTitle,
@@ -135,13 +148,35 @@ public class FurnitureLibraryEditor {
               return super.showOpenDialog(parentView, dialogTitle, contentType);
             }
           }
+
+          @Override
+          protected FileFilter [] getFileFilter(ContentType contentType) {
+            FileFilter [] fileFilter = super.getFileFilter(contentType);
+            if (contentType == ContentType.FURNITURE_LIBRARY) {
+              List<FileFilter> filters = new ArrayList<FileFilter>(fileFilter.length + 1);
+              filters.addAll(Arrays.asList(fileFilter));
+              filters.add(new FileFilter() {
+                  @Override
+                  public String getDescription() {
+                    return "Default library";
+                  }
+
+                  @Override
+                  public boolean accept(File f) {
+                    return f.isFile() && getFurnitureLibraryRecorder().isDefaultFurnitureLibrary(f.getAbsolutePath());
+                  }
+                });
+              fileFilter = filters.toArray(new FileFilter [filters.size()]);
+            }
+            return fileFilter;
+          }
         };
     }
     return this.contentManager;
   }
-  
+
   /**
-   * Returns a Swing view factory. 
+   * Returns a Swing view factory.
    */
   protected EditorViewFactory getViewFactory() {
     if (this.viewFactory == null) {
@@ -149,7 +184,7 @@ public class FurnitureLibraryEditor {
     }
     return this.viewFactory;
   }
-  
+
   /**
    * Returns the name of this application read from resources.
    */
@@ -161,25 +196,25 @@ public class FurnitureLibraryEditor {
    * Returns a new instance of an editor controller after <code>furnitureLibrary</code> was created.
    */
   protected EditorController createEditorController(FurnitureLibrary furnitureLibrary) {
-    return new EditorController(furnitureLibrary, getFurnitureLibraryRecorder(), getUserPreferences(), 
+    return new EditorController(furnitureLibrary, getFurnitureLibraryRecorder(), getUserPreferences(),
         getViewFactory(), getContentManager());
   }
 
   /**
    * Furniture Library Editor entry point.
-   * @param args may contain one .sh3f file to open, 
-   *     following a <code>-open</code> option.  
+   * @param args may contain one .sh3f file to open,
+   *     following a <code>-open</code> option.
    */
   public static void main(String [] args) {
     new FurnitureLibraryEditor().init(args);
   }
-  
+
   /**
    * Initializes application instance.
    */
   protected void init(final String [] args) {
-    initSystemProperties();    
-    initLookAndFeel();    
+    initSystemProperties();
+    initLookAndFeel();
     EventQueue.invokeLater(new Runnable() {
       public void run() {
         FurnitureLibraryEditor.this.run(args);
@@ -218,7 +253,7 @@ public class FurnitureLibraryEditor {
       UIManager.setLookAndFeel(System.getProperty("swing.defaultlaf", UIManager.getSystemLookAndFeelClassName()));
       // Change default titled borders under Mac OS X 10.5
       if (OperatingSystem.isMacOSXLeopardOrSuperior()) {
-        UIManager.put("TitledBorder.border", 
+        UIManager.put("TitledBorder.border",
             UIManager.getBorder("TitledBorder.aquaVariant"));
       }
       SwingTools.updateSwingResourceLanguage();
@@ -242,7 +277,7 @@ public class FurnitureLibraryEditor {
     final FurnitureLibrary furnitureLibrary = new FurnitureLibrary();
     final EditorController editorController = createEditorController(furnitureLibrary);
     final View editorView = editorController.getView();
-    
+
     final JFrame furnitureFrame = new JFrame() {
         {
           if (editorView instanceof JRootPane) {
@@ -252,7 +287,7 @@ public class FurnitureLibraryEditor {
           }
         }
       };
-    // Update frame image and title 
+    // Update frame image and title
     Image [] frameImages = {new ImageIcon(FurnitureLibraryEditor.class.getResource("resources/frameIcon.png")).getImage(),
                             new ImageIcon(FurnitureLibraryEditor.class.getResource("resources/frameIcon32x32.png")).getImage()};
     try {
@@ -276,11 +311,11 @@ public class FurnitureLibraryEditor {
     if (furnitureFrame.getJMenuBar() == null) {
       installAccelerators(furnitureFrame, editorController);
     }
-    
+
     if (OperatingSystem.isMacOSX()) {
       MacOSXConfiguration.bindToApplicationMenu(editorController);
     }
-  
+
     if (furnitureLibraryName != null) {
       editorController.open(furnitureLibraryName);
     }
@@ -301,7 +336,7 @@ public class FurnitureLibraryEditor {
   /**
    * Changes the input map of furniture library view to ensure accelerators work even with no menu.
    */
-  private void installAccelerators(JFrame furnitureFrame, 
+  private void installAccelerators(JFrame furnitureFrame,
                                    final EditorController furnitureLibraryController) {
     JComponent furnitureLibraryView = (JComponent)furnitureLibraryController.getView();
     InputMap inputMap = furnitureLibraryView.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
@@ -312,34 +347,34 @@ public class FurnitureLibraryEditor {
       }
     }
   }
-  
+
   /**
    * Updates <code>frame</code> title from <code>furnitureLibrary</code> name.
    */
-  private void updateFrameTitle(JFrame frame, 
+  private void updateFrameTitle(JFrame frame,
                                 FurnitureLibrary furnitureLibrary,
                                 UserPreferences  preferences,
                                 ContentManager   contentManager) {
     String furnitureLibraryLocation = furnitureLibrary.getLocation();
     String furnitureLibraryDisplayedName;
     if (furnitureLibraryLocation == null) {
-      furnitureLibraryDisplayedName = preferences.getLocalizedString(FurnitureLibraryEditor.class, "untitled"); 
+      furnitureLibraryDisplayedName = preferences.getLocalizedString(FurnitureLibraryEditor.class, "untitled");
     } else {
       furnitureLibraryDisplayedName = contentManager.getPresentationName(
           furnitureLibraryLocation, ContentManager.ContentType.FURNITURE_LIBRARY);
     }
-    
+
     String title = furnitureLibraryDisplayedName;
     if (OperatingSystem.isMacOSX()) {
-      // Use black indicator in close icon for a modified library 
+      // Use black indicator in close icon for a modified library
       Boolean furnitureLibraryModified = Boolean.valueOf(furnitureLibrary.isModified());
       // Set Mac OS X 10.4 property for backward compatibility
       frame.getRootPane().putClientProperty("windowModified", furnitureLibraryModified);
-      
+
       if (OperatingSystem.isMacOSXLeopardOrSuperior()) {
         frame.getRootPane().putClientProperty("Window.documentModified", furnitureLibraryModified);
-        
-        if (furnitureLibraryLocation != null) {        
+
+        if (furnitureLibraryLocation != null) {
           File furnitureLibraryFile = new File(furnitureLibraryLocation);
           if (furnitureLibraryFile.exists()) {
             // Update the icon in window title bar for library files
@@ -348,17 +383,17 @@ public class FurnitureLibraryEditor {
         }
       }
     } else {
-      title += " - " + getName(); 
+      title += " - " + getName();
       if (furnitureLibrary.isModified()) {
         title = "* " + title;
       }
     }
     frame.setTitle(title);
   }
-  
+
   /**
-   * Mac OS X configuration. The methods use in this class are invoked in a separated class 
-   * because they exist only under Mac OS X. 
+   * Mac OS X configuration. The methods use in this class are invoked in a separated class
+   * because they exist only under Mac OS X.
    */
   private static class MacOSXConfiguration {
     /**
@@ -366,38 +401,72 @@ public class FurnitureLibraryEditor {
      */
     public static void bindToApplicationMenu(final EditorController controller) {
       Application macosxApplication = Application.getApplication();
-      // Add a listener to Mac OS X application that will call controller methods
-      macosxApplication.addApplicationListener(new ApplicationAdapter() {      
-        @Override
-        public void handleQuit(ApplicationEvent ev) { 
-          controller.exit();
-        }
-        
-        @Override
-        public void handleAbout(ApplicationEvent ev) {
-          controller.about();
-          ev.setHandled(true);
-        }
+      boolean applicationListenerSupported = false;
+      try {
+        // Add a listener to Mac OS X application that will call
+        // controller methods of the active frame
+        // Call macosxApplication.addApplicationListener(new MacOSXApplicationListener(homeApplication, defaultController, defaultFrame));
+        macosxApplication.getClass().getMethod("addApplicationListener", Class.forName("com.apple.eawt.ApplicationListener")).invoke(macosxApplication,
+            MacOSXApplicationListener.class.getConstructor(EditorController.class).newInstance(controller));
+        macosxApplication.setEnabledAboutMenu(true);
+        macosxApplication.setEnabledPreferencesMenu(true);
+        applicationListenerSupported = true;
+      } catch (NoSuchMethodException ex) {
+      } catch (IllegalAccessException ex) {
+      } catch (InvocationTargetException ex) {
+      } catch (ClassNotFoundException ex) {
+      } catch (InstantiationException ex) {
+      }
 
-        @Override
-        public void handlePreferences(ApplicationEvent ev) {
-          controller.editPreferences();
-          ev.setHandled(true);
+      if (!applicationListenerSupported) {
+        // Probably running under Java 9 where previous methods were removed
+        if (OperatingSystem.isJavaVersionGreaterOrEqual("1.9")) {
+          try {
+            // Instantiate Java 9 handlers and set them on desktop by reflection
+            Class<?> desktopClass = Class.forName("java.awt.Desktop");
+            Object desktopInstance = desktopClass.getMethod("getDesktop").invoke(null);
+
+            Object quitHandler = new QuitHandler() {
+                public void handleQuitRequestWith(QuitEvent ev, QuitResponse answer) {
+                  MacOSXConfiguration.handleQuit(controller);
+                  answer.cancelQuit();
+                }
+              };
+            desktopClass.getMethod("setQuitHandler", QuitHandler.class).invoke(desktopInstance, quitHandler);
+
+            Object aboutHandler = new AboutHandler() {
+                public void handleAbout(AboutEvent ev) {
+                  MacOSXConfiguration.handleAbout(controller);
+                }
+              };
+            desktopClass.getMethod("setAboutHandler", AboutHandler.class).invoke(desktopInstance, aboutHandler);
+
+            Object preferencesHandler = new PreferencesHandler() {
+                public void handlePreferences(PreferencesEvent ev) {
+                  MacOSXConfiguration.handlePreferences(controller);
+                }
+              };
+            desktopClass.getMethod("setPreferencesHandler", PreferencesHandler.class).invoke(desktopInstance, preferencesHandler);
+
+            Object openFilesHandler = new OpenFilesHandler() {
+                public void openFiles(OpenFilesEvent ev) {
+                  for (java.io.File file : ev.getFiles()) {
+                    MacOSXConfiguration.handleOpenFile(controller, file.getAbsolutePath());
+                  }
+                }
+              };
+            desktopClass.getMethod("setOpenFileHandler", OpenFilesHandler.class).invoke(desktopInstance, openFilesHandler);
+
+            // Call Desktop.getDesktop().setQuitStrategy(QuitStrategy.CLOSE_ALL_WINDOWS) to prevent default call to System#exit
+            desktopClass.getMethod("setQuitStrategy", QuitStrategy.class).invoke(desktopInstance, QuitStrategy.CLOSE_ALL_WINDOWS);
+          } catch (NoSuchMethodException ex) {
+          } catch (IllegalAccessException ex) {
+          } catch (InvocationTargetException ex) {
+          } catch (ClassNotFoundException ex) {
+          }
         }
-        
-        @Override
-        public void handleOpenFile(ApplicationEvent ev) {
-          controller.open(ev.getFilename());
-        }
-        
-        @Override
-        public void handleReOpenApplication(ApplicationEvent ev) {
-          SwingUtilities.getWindowAncestor((JComponent)controller.getView()).toFront();
-        }
-      });
-      macosxApplication.setEnabledAboutMenu(true);
-      macosxApplication.setEnabledPreferencesMenu(true);
-      
+      }
+
       // Set application icon if program wasn't launch from bundle
       if (!"true".equalsIgnoreCase(System.getProperty("furniturelibraryeditor.bundle", "false"))) {
         try {
@@ -409,5 +478,79 @@ public class FurnitureLibraryEditor {
         }
       }
     }
+
+    /**
+     * Handles quit action.
+     */
+    private static void handleQuit(final EditorController controller) {
+      controller.exit();
+    }
+
+    /**
+     * Handles how about pane is displayed.
+     */
+    protected static void handleAbout(final EditorController controller) {
+      controller.about();
+    }
+
+    /**
+     * Handles how preferences pane is displayed.
+     */
+    private static void handlePreferences(final EditorController controller) {
+      controller.editPreferences();
+    }
+
+    /**
+     * Handles the opening of a document.
+     */
+    private static void handleOpenFile(EditorController controller, String fileName) {
+      controller.open(fileName);
+    }
+
+    /**
+     * Handles application when it's reopened.
+     */
+    private static void handleReOpenApplication(EditorController controller) {
+      SwingUtilities.getWindowAncestor((JComponent)controller.getView()).toFront();
+    }
+  }
+
+  /**
+   * Listener for Mac OS X application events.
+   * Declared in a separated static class and instantiated by reflection to be able to run Sweet Home 3D with Java 10.
+   */
+  private static class MacOSXApplicationListener extends ApplicationAdapter {
+    private final EditorController controller;
+
+    public MacOSXApplicationListener(EditorController controller) {
+      this.controller = controller;
+    }
+
+    @Override
+    public void handleQuit(ApplicationEvent ev) {
+      MacOSXConfiguration.handleQuit(this.controller);
+    }
+
+    @Override
+    public void handleAbout(ApplicationEvent ev) {
+      MacOSXConfiguration.handleAbout(this.controller);
+      ev.setHandled(true);
+    }
+
+    @Override
+    public void handlePreferences(ApplicationEvent ev) {
+      MacOSXConfiguration.handlePreferences(this.controller);
+      ev.setHandled(true);
+    }
+
+    @Override
+    public void handleOpenFile(ApplicationEvent ev) {
+      MacOSXConfiguration.handleOpenFile(this.controller, ev.getFilename());
+    }
+
+    @Override
+    public void handleReOpenApplication(ApplicationEvent ev) {
+      MacOSXConfiguration.handleReOpenApplication(this.controller);
+    }
   }
 }
